/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file main.c
 **
 ** This example demonstrates both interrupt and polling access mode of PPG timer.
 ** PPG wave can be observed at P40 during demonstration process.
 **
 ** History:
 **   - 2013-05-20  0.0.1  Edison Zhang        First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
#define  USER_BT             BT0
#define  PpgInitOutput()     SetPinFunc_TIOA0_0_OUT()

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
/*! \brief Interrupt count value */
static uint32_t m_u32CntIntTrg, m_u32CntIntUnder;

/*---------------------------------------------------------------------------*/
/* local functions prototypes                                                */
/*---------------------------------------------------------------------------*/
static void PpgUnderflowIntHandler(void);
static void PpgTrigIntHandler(void);
static void Delay(void);
/*---------------------------------------------------------------------------*/
/* global data                                                               */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* global functions                                                          */
/*---------------------------------------------------------------------------*/


/*!
 ******************************************************************************
 ** \brief PPG timer example code
 **
 ** 1. Initialization
 ** 3. Enable interrupt.
 ** 4. Enable count operation.
 ** 5. Start software trigger.
 ** 6. Disable interrupt
 ** 7. Restart software trigger
 ** 8. Disable count operation.
 ******************************************************************************
 */
int32_t main(void)
{
    stc_bt_ppg_config_t stcPpgConfig;
    stc_ppg_int_sel_t   stcPpgIntSel;
    stc_ppg_int_cb_t    stcPpgIntCallback;
    uint32_t u32Cnt=0;
    
#ifdef DEBUG_PRINT 
    printf("==================================================\n");
    printf("PPG Example Program Start \n");
    printf("==================================================\n");
#endif
  
    // Set Basetimer IO port  
    PpgInitOutput();
    
    // Set requested I/O mode 
    Bt_ConfigIOMode(&USER_BT, BtIoMode0);
    Bt_SelTimerMode(&USER_BT, BtPpgMode);
    
    // Initialize PPG timer
    stcPpgConfig.enPres = PpgPres1Div4;  // PPG clock = 5MHz @ PCLK = 20MHz
    stcPpgConfig.enMode = PpgContinuous; 
    stcPpgConfig.enExtTrig = PpgExtTrigDisable;
    stcPpgConfig.enOutputMask = PpgOutputNormal;
    stcPpgConfig.enOutputPolarity = PpgPolarityLow;
    stcPpgConfig.enRestartEn = PpgRestartEnable;
    Bt_Ppg_Init(&USER_BT, &stcPpgConfig);
    
    // Set cycle and duty value
    Bt_Ppg_WriteLowWidthVal(&USER_BT, 499);   // Cycle = (1+m)*PPG clock cycle = 100us
    Bt_Ppg_WriteHighWidthVal(&USER_BT, 499);  // Duty = (1+m)*PPG clock cycle = 100us
          
    // Enable Interrupt 
    stcPpgIntSel.bPpgTrigInt = 1;
    stcPpgIntSel.bPpgUnderflowInt = 1;
    stcPpgIntCallback.pfnPpgTrigIntCallback = PpgTrigIntHandler;
    stcPpgIntCallback.pfnPpgUnderflowIntCallback = PpgUnderflowIntHandler;
    Bt_Ppg_EnableInt(&USER_BT, &stcPpgIntSel, &stcPpgIntCallback);
    
    // Enable count operatoin
    Bt_Ppg_EnableCount(&USER_BT);
    
    // Start triggered by software
    Bt_Ppg_EnableSwTrig(&USER_BT);
    
    // Delay some time
    Delay();     

#ifdef DEBUG_PRINT 
    printf("Cycle: %d\n", 499);
    printf("Duty: %d\n", 499);       
#endif    
    
#ifdef DEBUG_PRINT 
    printf("==================================================\n");
    printf("Enable Interrupt mode, waiting several seconds approximately\n");
#endif  
    
    // Delay some time
    Delay(); 
    
#ifdef DEBUG_PRINT 
    printf("Interrupt times:\n"); 
    printf("PPG trigger interrupt:   %d\n", m_u32CntIntTrg);    
    printf("PPG underflow interrupt: %d\n", m_u32CntIntUnder);  
#endif
    
    // Disable Interrupt 
    Bt_Ppg_DisableInt(&USER_BT, &stcPpgIntSel);
       
#ifdef DEBUG_PRINT 
    printf("==================================================\n");
    printf("Disable Interrupt mode, waiting several seconds approximately\n");
#endif
    
    m_u32CntIntTrg = 0;
    m_u32CntIntUnder = 0;   
    
    // Delay some time
    Delay(); 
    
#ifdef DEBUG_PRINT 
    printf("Interrupt times:\n"); 
    printf("PPG trigger interrupt:   %d\n", m_u32CntIntTrg);          
    printf("PPG underflow interrupt: %d\n", m_u32CntIntUnder);  
#endif 
    
    // Restart triggered by software
    Bt_Ppg_EnableSwTrig(&USER_BT);
    
#ifdef DEBUG_PRINT 
    printf("==================================================\n");
    printf("Enable query mode, waiting several seconds approximately\n");
#endif
 
    m_u32CntIntTrg = 0;
    m_u32CntIntUnder = 0; 
    
    // Interrupt query mode 
    do 
    {   
        if (Bt_Ppg_GetIntFlag(&USER_BT, PpgTrigInt) == PdlSet)
        {
            Bt_Ppg_ClrIntFlag(&USER_BT, PpgTrigInt);
            m_u32CntIntTrg++;
        }
        else if (Bt_Ppg_GetIntFlag(&USER_BT, PpgUnderflowInt) == PdlSet)
        {
            Bt_Ppg_ClrIntFlag(&USER_BT, PpgUnderflowInt);
            m_u32CntIntUnder++;
        }
        u32Cnt++;
    } while(u32Cnt <2500000UL);
    
#ifdef DEBUG_PRINT
    printf("Interrupt request times:\n"); 
    printf("PPG trigger interrupt:   %d\n", m_u32CntIntTrg);         
    printf("PPG underflow interrupt: %d\n", m_u32CntIntUnder);  
#endif 
    
    // Disable PPG count
    Bt_Ppg_DisableCount(&USER_BT);
    
#ifdef DEBUG_PRINT 
    printf("==================================================\n");
    printf("PPG Example Program End \n");
    printf("==================================================\n");
#endif
    
    while(1);
}
/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

/*!
 ******************************************************************************
 ** \brief PPG underflow interrupt handler
 ******************************************************************************
 */
static void PpgUnderflowIntHandler(void)
{
    m_u32CntIntUnder++;
}

/*!
 ******************************************************************************
 ** \brief PPG trigger interrupt handler
 ******************************************************************************
 */
static void PpgTrigIntHandler(void)
{
    m_u32CntIntTrg++;
}

/*
 ******************************************************************************
 ** \brief Delay requested secondes approximately
 ******************************************************************************
 */
static void Delay(void)
{
    int32_t u8Cnt1,u8Cnt2;
   
    u8Cnt2 = SystemCoreClock;
     
    u8Cnt1 = 0;
    while (1) 
    {
        u8Cnt1++;
        if (u8Cnt1 > u8Cnt2) 
        {
            break;
        }
    }   
}

/*****************************************************************************/
/* END OF FILE */
