/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/******************************************************************************/
/** \file main.c
 **
 ** This example demonstrates both interrupt and polling access mode of PWM timer.
 ** PWM wave can be observed at P40 during demonstration process.
 **
 ** History:
 **   - 2013-05-20  0.0.1  Edison Zhang        First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "pdl_header.h"

/*---------------------------------------------------------------------------*/
/* local defines                                                             */
/*---------------------------------------------------------------------------*/
#define  USER_BT             BT0
#define  PwmInitOutput()     SetPinFunc_TIOA0_0_OUT()

/*---------------------------------------------------------------------------*/
/* local data                                                                */
/*---------------------------------------------------------------------------*/
/*! \brief Interrupt count value */
static uint32_t m_u32CntIntTrg, m_u32CntIntDuty, m_u32CntIntUnder;

/*---------------------------------------------------------------------------*/
/* local functions prototypes                                                */
/*---------------------------------------------------------------------------*/
static void PwmDutyMatchIntHandler(void);
static void PwmUnderflowIntHandler(void);
static void PwmTrigIntHandler(void);
static void Delay(void);
/*---------------------------------------------------------------------------*/
/* global data                                                               */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* global functions                                                          */
/*---------------------------------------------------------------------------*/


/*!
 ******************************************************************************
 ** \brief PWM timer example code
 **
 ** 1. Initialization
 ** 3. Enable interrupt.
 ** 4. Enable count operation.
 ** 5. Start software trigger.
 ** 6. Disable interrupt
 ** 7. Restart software trigger
 ** 8. Disable count operation.
 ******************************************************************************
 */
int32_t main(void)
{
    stc_bt_pwm_config_t stcPwmConfig;
    stc_pwm_int_sel_t   stcPwmIntSel;
    stc_pwm_int_cb_t    stcPwmIntCallback;
    uint32_t u32Cnt=0;
    
#ifdef DEBUG_PRINT 
    printf("==================================================\n");
    printf("PWM Example Program Start \n");
    printf("==================================================\n");
#endif
  
    // Set Basetimer IO port  
    PwmInitOutput();
    
    // Set requested I/O mode 
    Bt_ConfigIOMode(&USER_BT, BtIoMode0);
    Bt_SelTimerMode(&USER_BT, BtPwmMode);
    
    // Initialize PWM timer
    stcPwmConfig.enPres = PwmPres1Div4;  // PWM clock = 5MHz @ PCLK = 20MHz
    stcPwmConfig.enMode = PwmContinuous; 
    stcPwmConfig.enExtTrig = PwmExtTrigDisable;
    stcPwmConfig.enOutputMask = PwmOutputNormal;
    stcPwmConfig.enOutputPolarity = PwmPolarityLow;
    stcPwmConfig.enRestartEn = PwmRestartEnable;
    Bt_Pwm_Init(&USER_BT, &stcPwmConfig);
    
    // Set cycle and duty value
    Bt_Pwm_WriteCycleVal(&USER_BT, 999); // Cycle = (1+m)*PWM clock cycle = 200us
    Bt_Pwm_WriteDutyVal(&USER_BT, 199);  // Duty = (1+m)*PWM clock cycle = 40us
          
    // Enable Interrupt 
    stcPwmIntSel.bPwmTrigInt = 1;
    stcPwmIntSel.bPwmDutyMatchInt = 1;
    stcPwmIntSel.bPwmUnderflowInt = 1;
    stcPwmIntCallback.pfnPwmTrigIntCallback = PwmTrigIntHandler;
    stcPwmIntCallback.pfnPwmDutyMatchIntCallback = PwmDutyMatchIntHandler;
    stcPwmIntCallback.pfnPwmUnderflowIntCallback = PwmUnderflowIntHandler;
    Bt_Pwm_EnableInt(&USER_BT, &stcPwmIntSel, &stcPwmIntCallback);
    
    // Enable count operatoin
    Bt_Pwm_EnableCount(&USER_BT);
    
    // Start triggered by software
    Bt_Pwm_EnableSwTrig(&USER_BT);
    
    // Delay some time
    Delay();     

#ifdef DEBUG_PRINT 
    printf("Cycle: %d\n", 999);
    printf("Duty: %d\n", 199);       
#endif    
    
#ifdef DEBUG_PRINT 
    printf("==================================================\n");
    printf("Enable Interrupt mode, waiting several seconds approximately\n");
#endif  
    
    // Delay some time
    Delay(); 
    
    /* Disable Interrupt */ 
    Bt_Pwm_DisableInt(&USER_BT, &stcPwmIntSel);
    
#ifdef DEBUG_PRINT 
    printf("Interrupt times:\n"); 
    printf("PWM trigger interrupt:   %d\n", m_u32CntIntTrg);    
    printf("PWM duty match interrupt:%d\n", m_u32CntIntDuty);       
    printf("PWM underflow interrupt: %d\n", m_u32CntIntUnder);  
#endif
    
    
#ifdef DEBUG_PRINT 
    printf("==================================================\n");
    printf("Disable Interrupt mode, waiting several seconds approximately\n");
#endif
    
    m_u32CntIntTrg = 0;
    m_u32CntIntDuty = 0;
    m_u32CntIntUnder = 0;   
    
    // Delay some time
    Delay(); 
    
#ifdef DEBUG_PRINT 
    printf("Interrupt times:\n"); 
    printf("PWM trigger interrupt:   %d\n", m_u32CntIntTrg);    
    printf("PWM duty match interrupt:%d\n", m_u32CntIntDuty);       
    printf("PWM underflow interrupt: %d\n", m_u32CntIntUnder);
#endif 
    
    // Restart triggered by software
    Bt_Pwm_EnableSwTrig(&USER_BT);
    
#ifdef DEBUG_PRINT 
    printf("==================================================\n");
    printf("Enable query mode, waiting several seconds approximately\n");
#endif
 
    m_u32CntIntTrg = 0;
    m_u32CntIntDuty = 0;
    m_u32CntIntUnder = 0; 
    
    // Interrupt query mode 
    do 
    {   
        if (Bt_Pwm_GetIntFlag(&USER_BT, PwmTrigInt) == PdlSet)
        {
            Bt_Pwm_ClrIntFlag(&USER_BT, PwmTrigInt);
            m_u32CntIntTrg++;
        }
        else if (Bt_Pwm_GetIntFlag(&USER_BT, PwmDutyMatchInt) == PdlSet)
        {
            Bt_Pwm_ClrIntFlag(&USER_BT, PwmDutyMatchInt);
            m_u32CntIntDuty++;
        }
        else if (Bt_Pwm_GetIntFlag(&USER_BT, PwmUnderflowInt) == PdlSet)
        {
            Bt_Pwm_ClrIntFlag(&USER_BT, PwmUnderflowInt);
            m_u32CntIntUnder++;
        }
        u32Cnt++;
    } while(u32Cnt <2500000UL);
    
#ifdef DEBUG_PRINT
    printf("Interrupt request times:\n"); 
    printf("PWM trigger interrupt:   %d\n", m_u32CntIntTrg);    
    printf("PWM duty match interrupt:%d\n", m_u32CntIntDuty);       
    printf("PWM underflow interrupt: %d\n", m_u32CntIntUnder);
#endif 
    
    // Disable PWM count
    Bt_Pwm_DisableCount(&USER_BT);
    
#ifdef DEBUG_PRINT 
    printf("==================================================\n");
    printf("PWM Example Program End \n");
    printf("==================================================\n");
#endif
    
    // De-Initialize PWM 
    Bt_Pwm_DeInit(&USER_BT);

    while(1);
}
/*---------------------------------------------------------------------------*/
/* local functions                                                           */
/*---------------------------------------------------------------------------*/

/*!
 ******************************************************************************
 ** \brief PWM duty match interrupt handler
 ******************************************************************************
 */
static void PwmDutyMatchIntHandler(void)
{
    m_u32CntIntDuty++;
}

/*!
 ******************************************************************************
 ** \brief PWM underflow interrupt handler
 ******************************************************************************
 */
static void PwmUnderflowIntHandler(void)
{
    m_u32CntIntUnder++;
}

/*!
 ******************************************************************************
 ** \brief PWM trigger interrupt handler
 ******************************************************************************
 */
static void PwmTrigIntHandler(void)
{
    m_u32CntIntTrg++;
}

/*
 ******************************************************************************
 ** \brief Delay requested secondes approximately
 ******************************************************************************
 */
static void Delay(void)
{
    int32_t u8Cnt1,u8Cnt2;
   
    u8Cnt2 = SystemCoreClock;
     
    u8Cnt1 = 0;
    while (1) 
    {
        u8Cnt1++;
        if (u8Cnt1 > u8Cnt2) 
        {
            break;
        }
    }   
}

/*****************************************************************************/
/* END OF FILE */
